@@ -1,11 +1,13 @@
 import asyncio
 import logging
+import ssl
 from time import time
 from types import GeneratorType
-from typing import AsyncGenerator, Optional
+from typing import AsyncGenerator, Optional, Union
 from urllib.parse import urlparse
 
 from asynch.errors import (
+    PartiallyConsumedQueryError,
     ServerException,
     UnexpectedPacketFromServerError,
     UnknownPacketFromServerError,
@@ -13,14 +15,14 @@ from asynch.errors import (
 from asynch.proto import constants
 from asynch.proto.block import BlockStreamProfileInfo, ColumnOrientedBlock, RowOrientedBlock
 from asynch.proto.compression import get_compressor_cls
-from asynch.proto.context import Context
+from asynch.proto.context import Context, ExecuteContext
 from asynch.proto.cs import ClientInfo, QueryKind, ServerInfo
-from asynch.proto.io import BufferedReader, BufferedWriter
 from asynch.proto.progress import Progress
 from asynch.proto.protocol import ClientPacket, Compression, ServerPacket
 from asynch.proto.result import IterQueryResult, ProgressQueryResult, QueryInfo, QueryResult
 from asynch.proto.settings import write_settings
-from asynch.proto.streams.native import BlockInputStream, BlockOutputStream
+from asynch.proto.streams.block import BlockReader, BlockWriter
+from asynch.proto.streams.buffered import BufferedReader, BufferedWriter
 from asynch.proto.utils.escape import escape_params
 from asynch.proto.utils.helpers import chunks, column_chunks
 
@@ -72,7 +74,7 @@ class Connection:
         send_receive_timeout: int = constants.DBMS_DEFAULT_TIMEOUT_SEC,
         sync_request_timeout: int = constants.DBMS_DEFAULT_SYNC_REQUEST_TIMEOUT_SEC,
         compress_block_size: int = constants.DEFAULT_COMPRESS_BLOCK_SIZE,
-        compression: bool = False,
+        compression: Union[bool, str] = False,
         secure: bool = False,
         # Secure socket parameters.
         verify: bool = True,
@@ -81,6 +83,7 @@ class Connection:
         ciphers=None,
         alt_hosts: str = None,
         stack_track=False,
+        settings_is_important=False,
         **kwargs,
     ):
         self.stack_track = stack_track
@@ -94,12 +97,16 @@ class Connection:
                 url = urlparse("clickhouse://" + host)
                 self.hosts.append((url.hostname, url.port or default_port))
         self.database = database
+        self.host = None
+        self.port = None
         self.user = user
         self.password = password
         self.client_name = constants.DBMS_NAME + " " + client_name
         self.connect_timeout = connect_timeout
         self.send_receive_timeout = send_receive_timeout
         self.sync_request_timeout = sync_request_timeout
+        self.settings_is_important = settings_is_important
+        self._lock = asyncio.Lock()
         self.secure_socket = secure
         self.verify_cert = verify
 
@@ -129,50 +136,70 @@ class Connection:
         self.server_info: Optional[ServerInfo] = None
         self.context = Context()
         # Block writer/reader
-        self.block_in_stream: Optional[BlockInputStream] = None
-        self.block_out_stream: Optional[BlockOutputStream] = None
+        self.block_reader: Optional[BlockReader] = None
+        self.block_writer: Optional[BlockWriter] = None
+        self.block_reader_raw: Optional[BlockReader] = None
+        self.is_query_executing = False
+        self.client_trace_context = None
+
         self.settings = kwargs.pop("settings", {}).copy()
         self.client_settings = {
             "insert_block_size": int(
-                self.settings.pop("insert_block_size", constants.DEFAULT_INSERT_BLOCK_SIZE,)
+                self.settings.pop(
+                    "insert_block_size",
+                    constants.DEFAULT_INSERT_BLOCK_SIZE,
+                )
             ),
             "strings_as_bytes": self.settings.pop("strings_as_bytes", False),
             "strings_encoding": self.settings.pop("strings_encoding", constants.STRINGS_ENCODING),
+            "use_numpy": self.settings.pop("use_numpy", False),
+            "opentelemetry_traceparent": self.settings.pop("opentelemetry_traceparent", None),
+            "opentelemetry_tracestate": self.settings.pop("opentelemetry_tracestate", ""),
+            "quota_key": self.settings.pop("quota_key", ""),
         }
         self.last_query: Optional[QueryInfo] = None
         self.available_client_settings = (
             "insert_block_size",  # TODO: rename to max_insert_block_size
             "strings_as_bytes",
             "strings_encoding",
+            "use_numpy",
+            "opentelemetry_traceparent",
+            "opentelemetry_tracestate",
+            "quota_key",
         )
         self.context.settings = self.settings
         self.context.client_settings = self.client_settings
 
-    def get_block_in_stream(self):
+    def get_block_reader(self):
         if self.compression:
-            from asynch.proto.streams.compressed import CompressedBlockInputStream
+            from asynch.proto.streams.compressed import CompressedBlockReader
 
-            return CompressedBlockInputStream(self.reader, self.writer, self.context)
-        else:
-            from asynch.proto.streams.native import BlockInputStream
-
-            return BlockInputStream(self.reader, self.writer, self.context)
-
-    def get_block_out_stream(self):
-        if self.compression:
-            from .streams.compressed import CompressedBlockOutputStream
-
-            return CompressedBlockOutputStream(
+            return CompressedBlockReader(
                 self.reader,
                 self.writer,
-                self.compressor_cls,
-                self.compress_block_size,
                 self.context,
             )
         else:
-            from .streams.native import BlockOutputStream
+            from asynch.proto.streams.block import BlockReader
 
-            return BlockOutputStream(self.reader, self.writer, self.context)
+            return BlockReader(self.reader, self.writer, self.context)
+
+    def get_block_writer(self):
+        if self.compression:
+            compressor = self.compressor_cls(BufferedWriter())
+            from .streams.compressed import CompressedBlockWriter
+
+            return CompressedBlockWriter(
+                self.reader,
+                self.writer,
+                self.context,
+                compressor,
+                self.compress_block_size,
+            )
+        else:
+            from .streams.block import BlockWriter
+
+            return BlockWriter(self.reader, self.writer, self.context)
 
     async def send_hello(self):
         await self.writer.write_varint(ClientPacket.HELLO)
@@ -240,6 +267,25 @@ class Connection:
             message = self.unexpected_packet_message("Hello or Exception", packet_type)
             raise UnexpectedPacketFromServerError(message)
 
+    def _get_ssl_context(self):
+        if not self.secure_socket:
+            return None
+        ssl_ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+        ssl_version = self.ssl_options.get("ssl_version")
+        if ssl_version:
+            ssl_ctx.options |= ssl_version
+        ca_certs = self.ssl_options.get("ca_certs")
+        if ca_certs:
+            ssl_ctx.load_verify_locations(ca_certs)
+        else:
+            ssl_ctx.load_default_certs(ssl.Purpose.SERVER_AUTH)
+        ciphers = self.ssl_options.get("ciphers")
+        if ciphers:
+            ssl_ctx.set_ciphers(ciphers)
+        if self.verify_cert:
+            ssl_ctx.verify_mode = ssl.VerifyMode.CERT_REQUIRED
+        return ssl_ctx
+
     async def ping(self):
         await self.writer.write_varint(ClientPacket.PING)
         await self.writer.flush()
@@ -252,13 +298,13 @@ class Connection:
             raise UnexpectedPacketFromServerError(msg)
         return True
 
-    async def receive_data(self):
+    async def receive_data(self, raw=False):
         revision = self.server_info.revision
 
         if revision >= constants.DBMS_MIN_REVISION_WITH_TEMPORARY_TABLES:
             await self.reader.read_str()
 
-        return await self.block_in_stream.read()
+        return await (self.block_reader_raw if raw else self.block_reader).read()
 
     async def receive_exception(self):
         return await self.read_exception()
@@ -371,14 +417,20 @@ class Connection:
             packet.block = await self.receive_data()
 
         elif packet_type == ServerPacket.LOG:
-            block = await self.receive_data()
+            block = await self.receive_data(raw=True)
             self.log_block(block)
         elif packet_type == ServerPacket.END_OF_STREAM:
-            pass
-
+            self.is_query_executing = False
         elif packet_type == ServerPacket.TABLE_COLUMNS:
             packet.multistring_message = await self.receive_multistring_message(packet_type)
+        elif packet_type == ServerPacket.PART_UUIDS:
+            packet.block = await self.receive_data()
 
+        elif packet_type == ServerPacket.READ_TASK_REQUEST:
+            packet.block = await self.receive_data()
+
+        elif packet_type == ServerPacket.PROFILE_EVENTS:
+            packet.block = await self.receive_data(raw=True)
         else:
             await self.disconnect()
             raise UnknownPacketFromServerError(
@@ -415,7 +467,9 @@ class Connection:
 
     async def receive_progress(self):
         progress = Progress(self.reader)
-        await progress.read(self.server_info.revision,)
+        await progress.read(
+            self.server_info.revision,
+        )
         return progress
 
     async def send_cancel(self):
@@ -427,19 +481,24 @@ class Connection:
         await self.writer.write_str(query_id)
         revision = self.server_info.revision
         if revision >= constants.DBMS_MIN_REVISION_WITH_CLIENT_INFO:
-            client_info = ClientInfo(self.client_name)
+            client_info = ClientInfo(self.client_name, self.writer, self.context)
             client_info.query_kind = QueryKind.INITIAL_QUERY
-            await client_info.write(revision, self.writer)
+            await client_info.write(revision)
 
         settings_as_strings = (
             revision >= constants.DBMS_MIN_REVISION_WITH_SETTINGS_SERIALIZED_AS_STRINGS
         )
-        await write_settings(self.writer, self.context.settings, settings_as_strings)
-
+        await write_settings(
+            self.writer, self.context.settings, settings_as_strings, self.settings_is_important
+        )
+        if revision >= constants.DBMS_MIN_REVISION_WITH_INTERSERVER_SECRET:
+            await self.writer.write_str("")
         await self.writer.write_varint(QueryProcessingStage.COMPLETE)
         await self.writer.write_varint(self.compression)
 
-        await self.writer.write_str(query,)
+        await self.writer.write_str(
+            query,
+        )
 
         logger.debug("Query: %s", query)
 
@@ -447,20 +506,38 @@ class Connection:
 
     async def _init_connection(self, host: str, port: int):
         self.host, self.port = host, port
-        reader, writer = await asyncio.open_connection(host, port)
-        self.writer = BufferedWriter(writer, constants.BUFFER_SIZE)
-        self.reader = BufferedReader(reader, constants.BUFFER_SIZE)
-        self.block_in_stream = self.get_block_in_stream()
-        self.block_out_stream = self.get_block_out_stream()
+        reader, writer = await asyncio.open_connection(host, port, ssl=self._get_ssl_context())
+        self.writer = BufferedWriter(writer)
+        self.reader = BufferedReader(reader)
+        self.block_reader = self.get_block_reader()
+        self.block_reader_raw = BlockReader(self.reader, self.writer, self.context)
+        self.block_writer = self.get_block_writer()
+
         self.connected = True
         await self.send_hello()
         await self.receive_hello()
 
+    def reset_state(self):
+
+        self.writer = None
+        self.reader = None
+        self.block_reader = None
+        self.block_reader_raw = None
+        self.block_writer = None
+        self.connected = False
+
+        self.client_trace_context = None
+        self.server_info = None
+
+        self.is_query_executing = False
+
     async def disconnect(self):
         if self.connected:
             self.connected = False
             await self.writer.close()
 
+        self.reset_state()
+
     async def connect(self):
         if self.connected:
             await self.disconnect()
@@ -522,35 +599,101 @@ class Connection:
         """
 
         start_time = time()
-        self.make_query_settings(settings)
-        await self.force_connect()
-        self.last_query = QueryInfo(self.reader)
+        async with ExecuteContext(self, query, settings):
 
-        # INSERT queries can use list/tuple/generator of list/tuples/dicts.
-        # For SELECT parameters can be passed in only in dict right now.
-        is_insert = isinstance(args, (list, tuple, GeneratorType))
+            # INSERT queries can use list/tuple/generator of list/tuples/dicts.
+            # For SELECT parameters can be passed in only in dict right now.
+            is_insert = isinstance(args, (list, tuple, GeneratorType))
 
-        if is_insert:
-            rv = await self.process_insert_query(
+            if is_insert:
+                rv = await self.process_insert_query(
+                    query,
+                    args,
+                    external_tables=external_tables,
+                    query_id=query_id,
+                    types_check=types_check,
+                    columnar=columnar,
+                )
+            else:
+                rv = await self.process_ordinary_query(
+                    query,
+                    args,
+                    with_column_types=with_column_types,
+                    external_tables=external_tables,
+                    query_id=query_id,
+                    types_check=types_check,
+                    columnar=columnar,
+                )
+            self.last_query.store_elapsed(time() - start_time)
+            return rv
+
+    async def execute_with_progress(
+        self,
+        query,
+        params=None,
+        with_column_types=False,
+        external_tables=None,
+        query_id=None,
+        settings=None,
+        types_check=False,
+        columnar=False,
+    ):
+        """
+        Executes SELECT query with progress information.
+        See, :ref:`execute-with-progress`.
+
+        :param query: query that will be send to server.
+        :param params: substitution parameters for SELECT queries and data for
+                       INSERT queries. Data for INSERT can be `list`, `tuple`
+                       or :data:`~types.GeneratorType`.
+                       Defaults to ``None`` (no parameters  or data).
+        :param with_column_types: if specified column names and types will be
+                                  returned alongside with result.
+                                  Defaults to ``False``.
+        :param external_tables: external tables to send.
+                                Defaults to ``None`` (no external tables).
+        :param query_id: the query identifier. If no query id specified
+                         ClickHouse server will generate it.
+        :param settings: dictionary of query settings.
+                         Defaults to ``None`` (no additional settings).
+        :param types_check: enables type checking of data for INSERT queries.
+                            Causes additional overhead. Defaults to ``False``.
+        :param columnar: if specified the result will be returned in
+                         column-oriented form.
+                         Defaults to ``False`` (row-like form).
+        :return: :ref:`progress-query-result` proxy.
+        """
+
+        async with ExecuteContext(self, query, settings):
+            return await self.process_ordinary_query_with_progress(
                 query,
-                args,
-                external_tables=external_tables,
-                query_id=query_id,
-                types_check=types_check,
-                columnar=columnar,
-            )
-        else:
-            rv = await self.process_ordinary_query(
-                query,
-                args,
+                params=params,
                 with_column_types=with_column_types,
                 external_tables=external_tables,
                 query_id=query_id,
                 types_check=types_check,
                 columnar=columnar,
             )
-        self.last_query.store_elapsed(time() - start_time)
-        return rv
+
+    async def process_ordinary_query_with_progress(
+        self,
+        query,
+        params=None,
+        with_column_types=False,
+        external_tables=None,
+        query_id=None,
+        types_check=False,
+        columnar=False,
+    ):
+
+        if params is not None:
+            query = self.substitute_params(query, params)
+
+        await self.send_query(query, query_id=query_id)
+        await self.send_external_tables(external_tables, types_check=types_check)
+        return self.receive_result(
+            with_column_types=with_column_types, progress=True, columnar=columnar
+        )
 
     def make_query_settings(self, settings):
         settings = dict(settings or {})
@@ -568,7 +711,16 @@ class Connection:
         query_settings.update(settings)
         self.context.settings = query_settings
 
+    async def check_query_execution(self):
+        async with self._lock:
+            if self.is_query_executing:
+                raise PartiallyConsumedQueryError()
+
+            self.is_query_executing = True
+
     async def force_connect(self):
+        await self.check_query_execution()
+
         if not self.connected:
             await self.connect()
 
@@ -596,22 +748,22 @@ class Connection:
 
     async def send_external_tables(self, tables, types_check=False):
         for table in tables or []:
-            block = RowOrientedBlock(
-                self.writer, self.reader, table["structure"], table["data"], types_check=types_check
-            )
+            block = RowOrientedBlock(table["structure"], table["data"], types_check=types_check)
             await self.send_block(block, table_name=table["name"])
 
         # Empty block, end of data transfer.
-        await self.send_block(RowOrientedBlock(self.writer, self.reader))
+        await self.send_block(RowOrientedBlock())
 
     async def send_block(self, block, table_name=""):
         await self.writer.write_varint(ClientPacket.DATA)
 
         revision = self.server_info.revision
         if revision >= constants.DBMS_MIN_REVISION_WITH_TEMPORARY_TABLES:
-            await self.writer.write_str(table_name,)
+            await self.writer.write_str(
+                table_name,
+            )
 
-        await self.block_out_stream.write(block)
+        await self.block_writer.write(block)
 
     def substitute_params(self, query, params):
         if not isinstance(params, dict):
@@ -651,7 +803,8 @@ class Connection:
 
             elif packet.type == ServerPacket.EXCEPTION:
                 raise packet.exception
-
+            elif packet.type == ServerPacket.LOG:
+                self.log_block(packet.block)
             elif packet.type == ServerPacket.TABLE_COLUMNS:
                 pass
 
@@ -670,8 +823,6 @@ class Connection:
 
         for chunk in slicer(data, client_settings["insert_block_size"]):
             block = block_cls(
-                self.writer,
-                self.reader,
                 columns_with_types=sample_block.columns_with_types,
                 data=chunk,
                 types_check=types_check,
@@ -680,7 +831,7 @@ class Connection:
             inserted_rows += block.num_rows
 
         # Empty block means end of data.
-        await self.send_block(block_cls(self.writer, self.reader))
+        await self.send_block(block_cls())
         return inserted_rows
 
     async def iter_process_ordinary_query(
@@ -707,6 +858,11 @@ class Connection:
             for row in rows:
                 yield row
 
+    def track_current_database(self, query):
+        query = query.strip("; ")
+        if query.lower().startswith("use "):
+            self.database = query[4:].strip()
+
     async def execute_iter(
         self,
         query,
@@ -741,15 +897,13 @@ class Connection:
         :return: :ref:`iter-query-result` proxy.
         """
 
-        self.make_query_settings(settings)
-        await self.force_connect()
-        self.last_query = QueryInfo(self.reader)
+        async with ExecuteContext(self, query, settings):
 
-        return await self.iter_process_ordinary_query(
-            query,
-            params=params,
-            with_column_types=with_column_types,
-            external_tables=external_tables,
-            query_id=query_id,
-            types_check=types_check,
-        )
+            return await self.iter_process_ordinary_query(
+                query,
+                params=params,
+                with_column_types=with_column_types,
+                external_tables=external_tables,
+                query_id=query_id,
+                types_check=types_check,
+            )
