@@ -1,5 +1,5 @@
 from asynch.proto import constants
-from asynch.proto.io import BufferedReader
+from asynch.proto.streams.buffered import BufferedReader
 
 
 class Progress:
@@ -12,7 +12,8 @@ class Progress:
         self.reader = reader
 
     async def read(
-        self, server_revision,
+        self,
+        server_revision,
     ):
         self.rows = await self.reader.read_varint()
         self.bytes = await self.reader.read_varint()
